/*-------------------------------------------------------------------------+
| Copyright 2014 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.MappingEntryBase;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IExecutionUnitAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IRequestAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.IResourceAdapter;

/**
 * Base class for {@link TaskMappingEntry}s representing the one allocation of an
 * {@link ITaskAdapter} to an {@link IExecutionUnitAdapter}.
 * 
 * @author barner
 */
public abstract class TaskMappingEntry<S extends IRequestAdapter<?>, T extends IResourceAdapter<?>>
		extends MappingEntryBase<S, T> {

	/** Constructs a mapping entry the represents the mapping to a given deployment target. */
	protected TaskMappingEntry(S deployableComponent, T target) {
		super(deployableComponent, target);
	}

	/** {@inheritDoc} */
	@Override
	public T getTarget() {
		return target;
	}

	/**
	 * Sets target {@link IExecutionUnitAdapter} to the given {@link IExecutionUnitAdapter}
	 * overriding the previous target reference.
	 * 
	 * @param target
	 *            {@link IExecutionUnitAdapter} defining the allocation target.
	 */
	public void resetTarget(T target) {
		this.target = target;
	}

	/** {@inheritDoc} */
	@Override
	public S getSource() {
		return source;
	}

	/** {@inheritDoc} */
	@Override
	public abstract TaskMappingEntry<S, T> clone();
}
