/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.evaluate;

import java.util.Collection;
import java.util.Set;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.CompositePhenotype;
import org.opt4j.core.Objectives;
import org.opt4j.core.Value;
import org.opt4j.core.genotype.CompositeGenotype;
import org.opt4j.core.problem.Evaluator;

/**
 * Interface for evaluators of {@link CompositePhenotype}s. It defines a 'merge' method that each
 * concrete {@link CompositeEvaluator} must implement and which describes how the single
 * {@link ComposablePhenotype}s are merged into a {@link CompositePhenotype}.
 * 
 * @author diewald
 */
public interface CompositeEvaluator<P> extends Evaluator<P> {

	/**
	 * Adds a {@link ComposableEvaluator} for the given {@link ComposablePhenotype} class. The
	 * evaluation will be triggered automatically by the evaluation of the {@link CompositeGenotype}
	 * and user defined merge function will consider the resulting {@link Objectives} of this
	 * {@link ComposableEvaluator}, if desired.
	 */
	<S extends ComposablePhenotype<?>> void registerSubEvaluator(Class<S> subPhenotype,
			ComposableEvaluator<S> subEvaluator) throws Exception;

	/**
	 * Merges the evaluations of {@link ComposablePhenotype}s into an evaluation of the overall
	 * {@link CompositeGenotype}.
	 */
	Value<?> mergeSubEvaluations(Collection<Value<?>> results);

	/**
	 * Returns the collection of {@link ComposableEvaluator}s registered for this
	 * {@link ComposableEvaluator}.
	 */
	Collection<ComposableEvaluator<? extends ComposablePhenotype<?>>> getSubEvaluators();

	/**
	 * Returns the collection of {@link ComposableEvaluator}s registered for this
	 * {@link ComposableEvaluator} and all its children. Recursive version of
	 * {@link #getSubEvaluators()}.
	 */
	Set<ComposableEvaluator<? extends ComposablePhenotype<?>>> getAllSubEvaluators();
}
