/*-------------------------------------------------------------------------+
| Copyright 2015 fortiss GmbH                                              |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.backend.opt4j.solution;

import java.util.ArrayList;
import java.util.Collection;
import java.util.LinkedList;
import java.util.Queue;

import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.ArchitectureExplorationEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.ComposablePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.CompositePhenotype;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.CompositePhenotypeBase;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;

/**
 * Base class for solutions (=> {@link Phenotype}s) of an architecture exploration problem.
 * 
 * @author diewald
 */
// TODO: Check commonalities of this class with the FlatPhenotypeMap.
// TODO: Check whether it would be beneficial to create a IExplorationSolutionBase Class.
public class ArchitectureSolution extends CompositePhenotypeBase<ArchitectureExplorationEncoding>
		implements IExplorationSolution<ArchExpSubSolution<?>> {

	/** {@inheritDoc} */
	@SuppressWarnings("unchecked")
	@Override
	public Collection<Class<? extends ArchExpSubSolution<?>>> getAllSubSolutionClasses() {
		Collection<Class<? extends ArchExpSubSolution<?>>> subSolutionClasses = new ArrayList<>();
		Queue<ComposablePhenotype<?>> traversalList = new LinkedList<>();
		traversalList
				.addAll((Collection<? extends ComposablePhenotype<?>>)(Collection<?>)subPhenotypes
						.values());
		while(!traversalList.isEmpty()) {
			ComposablePhenotype<?> phenotype = traversalList.poll();
			subSolutionClasses
					.add((Class<? extends ArchExpSubSolution<?>>)phenotype.getPTypeToRegister());
			if(phenotype instanceof CompositePhenotype) {
				traversalList.addAll(((CompositePhenotype<?>)phenotype).getSubPhenotypes());
			}
		}
		return subSolutionClasses;
	}

	/** {@inheritDoc} */
	@Override
	public <T extends Phenotype> T getSubSolution(Class<T> solutionClass) {
		Queue<Phenotype> traversalList = new LinkedList<>();
		traversalList.addAll(subPhenotypes.values());
		while(!traversalList.isEmpty()) {
			Phenotype phenotype = traversalList.poll();

			if(solutionClass.isAssignableFrom(phenotype.getClass())) {
				@SuppressWarnings("unchecked") T retPheno = (T)phenotype;
				return retPheno;
			}

			if(phenotype instanceof CompositePhenotype) {
				traversalList.addAll(((CompositePhenotype<?>)phenotype).getSubPhenotypes());
			}
		}
		return null;
	}
}
