/*-------------------------------------------------------------------------+
| Copyright 2017 fortiss GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.af3.exploration.alg.dse.evaluator.constraint;

import org.eclipse.emf.ecore.EObject;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.Partition;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.partitionmapping.PartitionMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEncoding;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.encoding.taskmapping.TaskMappingEntry;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.FlatPhenotypeMap;
import org.fortiss.af3.exploration.alg.dse.backend.opt4j.extensions.compositegene.phenotype.Phenotype;
import org.fortiss.af3.exploration.alg.dse.evaluator.TaskMappingEvaluator;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.ITaskAdapter;
import org.fortiss.af3.exploration.alg.dse.sysmodel.arch.SystemModelAdapter;
import org.fortiss.af3.exploration.alg.service.IExplorationTargetEvaluator;
import org.fortiss.af3.exploration.dsl_v2.model.function.Maximize;
import org.fortiss.af3.exploration.dsl_v2.model.function.Minimize;
import org.fortiss.af3.exploration.model.ExplorationConstraint;
import org.fortiss.af3.exploration.model.ExplorationTarget;
import org.opt4j.core.IntegerValue;
import org.opt4j.core.Value;

import com.google.common.collect.ClassToInstanceMap;

/**
 * 
 * @author diewald
 */
public class MappingEvaluatorConstraint extends TaskMappingEvaluator implements
		IConstraintEvaluator<TaskMappingEncoding<ITaskAdapter<?>, TaskMappingEntry<ITaskAdapter<?>, Partition>>, Boolean, ExplorationConstraint<Boolean>> {

	/** Prototype constructor. */
	public MappingEvaluatorConstraint() {
	}

	/**
	 * Constructor for execution: Defines the systemModelAdapter abstraction and the
	 * {@link ExplorationTarget} to evaluate.
	 */
	public MappingEvaluatorConstraint(SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter,
			ExplorationTarget<Double> explorationTarget) throws Exception {
		super(systemModelAdapter, explorationTarget);
	}

	/** {@inheritDoc} */
	@Override
	public Value<?> evaluateGoal(
			TaskMappingEncoding<ITaskAdapter<?>, TaskMappingEntry<ITaskAdapter<?>, Partition>> phenotype,
			FlatPhenotypeMap<Phenotype> phenotypeTypeMap,
			ClassToInstanceMap<EObject> transformedModels) {
		mapping = phenotype;
		virtMapping = phenotypeTypeMap.getEncoding(PartitionMappingEncoding.class);
		assert (virtMapping != null);

		Boolean retVal = null;
		// FIXME: pass parsing exception to upper layers.
		try {
			retVal = expression.accept(this, Boolean.class);
		} catch(Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			return new IntegerValue(Integer.MIN_VALUE);
		}

		// TODO Auto-generated method stub
		return new IntegerValue(retVal == true ? 1 : 0);
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Maximize max) throws Exception {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public Object visit(Minimize min) throws Exception {
		// TODO Auto-generated method stub
		return null;
	}

	/** {@inheritDoc} */
	@Override
	public IExplorationTargetEvaluator<TaskMappingEncoding<ITaskAdapter<?>, TaskMappingEntry<ITaskAdapter<?>, Partition>>, Double, ExplorationTarget<Double>>
			createEvaluator(SystemModelAdapter<?, ?, ?, ?, ?, ?, ?> systemModelAdapter,
					ExplorationTarget<Double> explorationTarget)
					throws InstantiationException, Exception {
		return new MappingEvaluatorConstraint(systemModelAdapter, explorationTarget);
	}

	/** {@inheritDoc} */
	@Override
	public <M> boolean validate(ExplorationConstraint<?> expTarget, M solution,
			Class<M> solutionType, Value<?> value) {
		// FIXME: For porting
		if(value == null) {
			return true;
		}

		Double evalValue = value.getDouble();
		if(evalValue > 0.0) {
			return false;
		}
		return true;
	}
}
